////////////////////////////////////////////////////////////////////////////////
//
//		Licensed Materials - Property of IBM
//  		xxxx-xxx
//		(C) Copyright IBM Corp. 2004. All rights reserved.
//		US Government Users Restricted Rights - Use, duplication or disclosure
//		restricted by GSA ADP Schedule Contract with IBM Corp.
//
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has been
// deposited with the U.S. Copyright Office.
//
////////////////////////////////////////////////////////////////////////////////
package com.ibm.debug.idebug.engine;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.osgi.service.datalocation.Location;
import org.eclipse.osgi.service.environment.Constants;
import org.osgi.framework.Bundle;

/**
 * Class used to locate PICL engine files.
 */
public class PiclEngineFileLocator {
	private static String[]		engineSearchPath	= null;

	/**
	 * Used to create a new instance of this class.
	 */
	public PiclEngineFileLocator() {
		if (engineSearchPath == null)
			engineSearchPath = buildEngineSearchPath();
	}

	/**
	 * Used to find the specified engine file by searching through the engine
	 * search path.
	 * 
	 * @param filename
	 *            The engine file to search for.
	 * @return Returns the requested file.
	 */
	public File findFile(String filename) {
		if (engineSearchPath == null)
			return null;

		for (int i = 0; i < engineSearchPath.length; i++) {
			String path = engineSearchPath[i];
			if (path == null)
				continue;

			File file = new File(path + File.separatorChar + filename);
			try {
				if (file.exists() && file.isFile())
					return file;
			} catch (SecurityException e) {
			}
		}

		return null;
	}

	/**
	 * Used to get the search path used to find engine files.
	 * 
	 * @return Returns the engine search path.
	 */
	public String[] getSearchPath() {
		return engineSearchPath;
	}

	/**
	 * Used to build the search path used to find engine files.
	 * 
	 * @return An array of paths that are searched when looking for engine
	 *         files.
	 */
	protected String[] buildEngineSearchPath() {
		String[] searchPaths = null;
		ArrayList paths = new ArrayList();

		// lets start looking from the root of the debugger installation
		// directory
		Location installLocation = Platform.getInstallLocation();
		URL installURL = installLocation != null ? installLocation.getURL()
			: null;
		if (installURL != null) {
			StringBuffer temp = new StringBuffer(installURL.getPath());
			temp.append(File.separator).append("..");
			temp.append(File.separator).append("bin");

			IPath installPath = new Path(temp.toString());
			paths.add(installPath.toOSString());
		}

		// the engine could also exist as a fragment of this plug-in
		Bundle engineBundle = Platform.getBundle(EngineConstants.PLUGIN_NAME);
		if (engineBundle != null) {
			StringBuffer temp = new StringBuffer(Platform.getOS());
			temp.append(File.separator).append(Platform.getOSArch());
			temp.append(File.separator).append("engine");
			temp.append(File.separator).append("bin");

			IPath partialFragmentPath = new Path(temp.toString());
			URL fragmentURL = Platform.find(engineBundle, partialFragmentPath,
				null);
			if (fragmentURL != null) {
				try {
					URL localURL = Platform.asLocalURL(fragmentURL);
					IPath fragmentPath = new Path(localURL.getPath());
					paths.add(fragmentPath.toOSString());
				} catch (IOException e) {
				}
			}
		}
		
		// we should also search using the environment path variable
		String envPath = System.getProperty("java.library.path");
		if (envPath != null) {
			paths.addAll(Arrays.asList(envPath.split(File.pathSeparator)));
		}

		// finally check /usr/bin directory on UNIX
		if (!Platform.getOS().equals(Constants.OS_WIN32))
			paths.add("/usr/bin");

		searchPaths = (String[]) paths.toArray(new String[paths.size()]);
		return searchPaths;
	}
	
	private static final String	IBMCopyRight	= "(C) Copyright IBM Corp. 2004. All rights reserved.";
}